/*****************************************************************************
 *
 * MODULE:             OS Tutorial Application
 *
 * COMPONENT:          OSTutorial.h
 * DESCRIPTION:        OS Tutorial Application
 *
*/
/****************************************************************************
*
* This software is owned by NXP B.V. and/or its supplier and is protected
* under applicable copyright laws. All rights are reserved. We grant You,
* and any third parties, a license to use this software solely and
* exclusively on NXP products [NXP Microcontrollers such as JN5148, JN5142, JN5139].
* You, and any third parties must reproduce the copyright and warranty notice
* and any other legend of ownership on each copy or partial copy of the
* software.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.

* Copyright NXP B.V. 2012. All rights reserved
*
***************************************************************************/

#ifndef OS_TUTORIAL_H_
#define OS_TUTORIAL_H_

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#define LED0    (0)
#define LED1    (1)

#ifdef DK4
/*
 * Define the IO pins we are using on the DK4 fitted with Generic Expansion Board
 */
#define LED_0_PIN    (16)
#define LED_1_PIN    (13)

#define SW_0_PIN        (11)
#define SW_1_PIN        (12)
#define BUTTON_MASK    ((1<<SW_0_PIN) | (1<<SW_1_PIN))

#define LED_INIT()           vGenericLEDInit()
#define SET_LED(m, n)        vGenericLEDSetOutput( (1<<m), n )

#define INFO_STRING         "Built for DK4 fitted with Generic Expansion Board\n"

#else
/*
 * Define the IO pins we are using on the DK2 platform
 */
#define LED_0_PIN    (16)
#define LED_1_PIN    (17)

#define SW_0_PIN    (9)
#define SW_1_PIN    (10)
#define BUTTON_MASK    ((1<<SW_0_PIN) | (1<<SW_1_PIN))

#define LED_INIT()          vLedInitRfd()
#define SET_LED(m, n)       vLedControl( m, n )

#define INFO_STRING         "Built for DK2 Sensor Board\n"

#endif
#define ON      (1)
#define OFF     (0)

#define FLASH_TIME_0    APP_TIME_MS(1000)
#define FLASH_TIME_1    APP_TIME_MS(360)

#define BUTTON_SCAN_TIME    APP_TIME_MS(10)

#define SW_0_PRESSED	(1)
#define SW_1_PRESSED	(2)

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/



/****************************************************************************/
/***        External Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/****************************************************************************/
/****************************************************************************/

#endif /*OS_TUTORIAL_H_*/
