/*****************************************************************************
 *
 * MODULE:             Home Sensor Demo
 *
 * COMPONENT:          app_start.c
 *
 * AUTHOR:             MRW
 *
 * DESCRIPTION:        Startup the home sensor demo application
 *
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_sware/Projects/User_Doc/User_Guides/UG3075/Trunk/Examples/JN-UG-3075-OS-Tutorial/OSTutorialFive/Source/app_startFive.c $
 *
 * $Revision: 49860 $
 *
 * $LastChangedBy: nxp29772 $
 *
 * $LastChangedDate: 2012-11-23 13:47:54 +0000 (Fri, 23 Nov 2012) $
 *
 * $Id: app_startFive.c 49860 2012-11-23 13:47:54Z nxp29772 $
 *
 *****************************************************************************
 *
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2009. All rights reserved
 *
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include "os.h"
#include "os_gen.h"
#include "dbg.h"
#include "dbg_uart.h"
#include "pwrm.h"

#include "appapi.h"
#ifdef DK4
#include "GenericBoard.h"
#else
#include "ledcontrol.h"
#endif


#include "app_os_tutorial.h"
#include "app_timer_driver.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#ifndef TRACE_APP
#define TRACE_APP	TRUE
#endif

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE void vInitialiseApp(void);
PRIVATE void vUnclaimedInterrupt(void);
PRIVATE void vOSError(OS_teStatus eStatus, void *hObject);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/


/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/



/****************************************************************************
 *
 * NAME: vAppMain
 *
 * DESCRIPTION:
 * Entry point for application from a cold start.
 *
 * RETURNS:
 * Never returns.
 *
 ****************************************************************************/

extern void *stack_low_water_mark;

PUBLIC void vAppMain(void)
{
    // Initialise uart for debug
    DBG_vUartInit(DBG_E_UART_0, DBG_E_UART_BAUD_RATE_115200);

    DBG_vPrintf(TRUE, "\n\nPower Up!\n");
    DBG_vPrintf(TRACE_APP, INFO_STRING);

    /*
     * Initialise the stack overflow exception to trigger if the end of the
     * stack is reached. See the linker command file to adjust the allocated
     * stack size.
     */
    vAHI_SetStackOverflow(TRUE, (uint32)&stack_low_water_mark);

    // Stop the watchdog
    vAHI_WatchdogStop();

	u32AppApiInit(NULL, NULL, NULL, NULL, NULL, NULL);

    OS_vStart(vInitialiseApp, vUnclaimedInterrupt, vOSError);

	/* idle task commences on exit from OS start call */
    while (TRUE) {
        PWRM_vManagePower();
        //DBG_vPrintf(TRACE_APP, "Idle Loop\n");
    }
}

void vAppRegisterPWRMCallbacks(void)
{
    // Never sleeps so
    // No call backs to register
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/****************************************************************************
 *
 * NAME: vInitialiseApp
 *
 * DESCRIPTION:
 * Initialises Zigbee stack, hardware and application.
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void vInitialiseApp(void)
{
    /* initialise JenOS modules */
	DBG_vPrintf(TRACE_APP, "Initialising PWRM ... ");
    PWRM_vInit(E_AHI_SLEEP_OSCON_RAMON);

    // Stop it ever sleeping
    PWRM_eStartActivity();
	DBG_vPrintf(TRACE_APP, "Done\n");

	// Initialise the Leds
	LED_INIT();
	SET_LED(LED0, OFF);
	SET_LED(LED1, OFF);

	/* Set DIO lines to inputs with buttons connected (Rfd) */
    vAHI_DioSetDirection(BUTTON_MASK, 0);

    /* Set the edge detection for falling edges */
    vAHI_DioWakeEdge(0, BUTTON_MASK);

    /* Enable interrupts to occur on selected edge */
    vAHI_DioWakeEnable(BUTTON_MASK, 0);

    /* start led 0 flash timer */
    DBG_vPrintf(TRACE_APP, "Start APP_tmrFlashLed0... \n");
	OS_eStartSWTimer(APP_tmrFlashLed0, FLASH_TIME_0, NULL);

	/* start led 1 flash timer */
    DBG_vPrintf(TRACE_APP, "Start APP_tmrFlashLed1... \n");
    OS_eStartSWTimer(APP_tmrFlashLed1, FLASH_TIME_1, NULL);

}

/****************************************************************************
 *
 * NAME: vUnclaimedInterrupt
 *
 * DESCRIPTION:
 * Catches any unexpected interrupts
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void vUnclaimedInterrupt(void)
{
	register uint32 u32PICSR, u32PICMR;

	asm volatile ("l.mfspr %0,r0,0x4800" :"=r"(u32PICMR) : );
	asm volatile ("l.mfspr %0,r0,0x4802" :"=r"(u32PICSR) : );

	DBG_vPrintf(TRACE_APP, "Unclaimed interrupt : %x : %x\n", u32PICSR,u32PICSR);

	while (1);
}

/****************************************************************************
 *
 * NAME: vOSError
 *
 * DESCRIPTION:
 * Catches any unexpected OS errors
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void vOSError(OS_teStatus eStatus, void *hObject)
{
    OS_thTask hTask;

    if (eStatus != OS_E_QUEUE_EMPTY && ((eStatus < OS_E_SWTIMER_STOPPED) || (eStatus > OS_E_SWTIMER_RUNNING) ))
    {

    	DBG_vPrintf(TRACE_APP, "OS Error %d, offending object handle = 0x%08x\n", eStatus, hObject);

    	/* NB the task may have been pre-empted by an ISR which may be at fault */
    	OS_eGetCurrentTask(&hTask);
    	DBG_vPrintf(TRACE_APP, "Currently active task handle = 0x%08x\n", hTask);
#ifdef OS_STRICT_CHECKS
    	DBG_vPrintf(TRACE_APP, "Currently active ISR fn address = 0x%08x\n", OS_prGetActiveISR());
#endif

    	while(1);
    }
}

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
