/*****************************************************************************
 *
 * MODULE:             OS Tutorial
 *
 * COMPONENT:          app_os_tutorial.c
 *
 * AUTHOR:             RCLAY
 *
 * DESCRIPTION:        OS TUrutorial Application
 *
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_sware/Projects/User_Doc/User_Guides/UG3075/Trunk/Examples/JN-UG-3075-OS-Tutorial/OSTutorialFour/Source/app_os_tutorialFour.c $
 *
 * $Revision: 49860 $
 *
 * $LastChangedBy: nxp29772 $
 *
 * $LastChangedDate: 2012-11-23 13:47:54 +0000 (Fri, 23 Nov 2012) $
 *
 * $Id: app_os_tutorialFour.c 49860 2012-11-23 13:47:54Z nxp29772 $
 *
 *****************************************************************************
 *
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2009. All rights reserved
 *
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include "os.h"
#include "os_gen.h"
#include "dbg.h"
#include "AppHardwareApi.h"
#include "app_timer_driver.h"
#include "app_os_tutorial.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/


#ifndef TRACE_APP
#define TRACE_APP	TRUE
#endif


/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/
PRIVATE void vWriteToPortPin(uint8 u8PinNo, bool bState);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/
PRIVATE const uint8 u8ButtonPins[2] = {SW_0_PIN, SW_1_PIN};

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/


/****************************************************************************
 *
 * NAME: APP_taskLed0
 *
 * DESCRIPTION:
 * Task to flash Led 0
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
OS_TASK(APP_taskLed0)
{
    static bool bLedState = 0;
    DBG_vPrintf(TRACE_APP, "Led Task0\n");

    // Prolong the execution period of the task
    volatile unsigned int i;
    for (i=0; i<0x7ffff; i++);

    // Toggle the led
    bLedState = ~bLedState;

    // Set the led
    OS_eEnterCriticalSection(mutexOutPort);
    vWriteToPortPin(LED_0_PIN, bLedState);
    OS_eExitCriticalSection(mutexOutPort);


    /* flash again in 1 second */
    OS_eContinueSWTimer(APP_tmrFlashLed0, FLASH_TIME_0, NULL);

    DBG_vPrintf(TRACE_APP, "Terminating task0\n");

}

/****************************************************************************
 *
 * NAME: APP_taskLed1
 *
 * DESCRIPTION:
 * Task to flash Led 1
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
OS_TASK(APP_taskLed1)
{
    static bool bLedState = 0;
    static enum {
        E_FLASH,
        E_NO_FLASH
    }eState = E_FLASH;
    DBG_vPrintf(TRACE_APP, "Led Task1\n");

    uint8 u8Button = 0;
    if (OS_E_OK != OS_eCollectMessage(APP_msgButtons, (void*)&u8Button)) {
        //DBG_vPrintf(TRACE_APP, "No event to process!\n");
    } else {
        DBG_vPrintf(TRACE_APP, "Collect Button %d\n", u8Button);
        if (u8Button == SW_0_PRESSED) {
            eState = E_FLASH;
        } else if (u8Button == SW_1_PRESSED) {
            eState = E_NO_FLASH;
            OS_eEnterCriticalSection(mutexOutPort);
            vWriteToPortPin(LED_1_PIN, 1);
            OS_eExitCriticalSection(mutexOutPort);
        }
    }

    if (eState == E_FLASH) {
        // Toggle the led
        bLedState = ~bLedState;

        // Set the led
        OS_eEnterCriticalSection(mutexOutPort);
        vWriteToPortPin(LED_1_PIN, bLedState);
        OS_eExitCriticalSection(mutexOutPort);

    }

    /* flash again in 360ms */
    OS_eContinueSWTimer(APP_tmrFlashLed1, FLASH_TIME_1, NULL);


    DBG_vPrintf(TRACE_APP, "Terminating task1\n");

}

/****************************************************************************
 *
 * NAME: APP_taskButtonScan
 *
 * DESCRIPTION:
 * Scan buttons and debounce
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
OS_TASK(APP_taskButtonScan)
{
    static uint8 u8Debounce[2] = {0xff,0xff};
    static uint8 u8ButtonState[2] = {0,0};
    /*
     * The DIO changed status register is reset here before the scan is performed.
     * This avoids a race condition between finishing a scan and re-enabling the
     * DIO to interrupt on a falling edge.
     */
    (void)u32AHI_DioWakeStatus();

    uint8 u8AllReleased = 0xff;
    unsigned int i;
    uint32 u32DIOState = u32AHI_DioReadInput() & BUTTON_MASK;

    for (i = 0; i < 2; i++) {
        uint8 u8Button = (uint8)((u32DIOState >> u8ButtonPins[i]) & 1);

        u8Debounce[i] <<= 1;
        u8Debounce[i] |= u8Button;
        u8AllReleased &= u8Debounce[i];

        if ( (0 == u8Debounce[i]) && !u8ButtonState[i]) {
            u8ButtonState[i] = TRUE;

            /*
             * button consistently depressed for 8 scan periods
             * so post message to application task to indicate
             * a button down event
             */
            uint8 u8Pressed  = i+1;
            DBG_vPrintf(TRACE_APP, "Post Button %d\n", u8Pressed);

            OS_ePostMessage(APP_msgButtons, (void*)&u8Pressed);
        } else if ((0xff == u8Debounce[i]) && (u8ButtonState[i] != FALSE)) {
            u8ButtonState[i] = FALSE;

            //DBG_vPrintf(TRACE_APP, "Released %i\n", i+1);
        }

    }

    if (0xff == u8AllReleased) {
        /*
         * all buttons high so set dio to interrupt on change
         */
        DBG_vPrintf(TRACE_APP, "ALL UP\n", i);
        vAHI_DioWakeEnable(BUTTON_MASK, 0);
    } else {
        /*
         * one or more buttons is still depressed so continue scanning
         */
        OS_eContinueSWTimer(APP_tmrButtonScan, BUTTON_SCAN_TIME, NULL);
    }
}

/****************************************************************************
 *
 * NAME: APP_isrSystem Controller
 *
 * DESCRIPTION:
 * ISR - Handles interrupts from Comparators, Wake Timers and Dio pins
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
OS_ISR(APP_isrSystemController)
{
    /*
     * ISR handles Wake Timer, Comparator and Dio interrupts
     */
    // Clear pending interrupt flags
    (void)u8AHI_WakeTimerFiredStatus();
    (void)u8AHI_ComparatorWakeStatus();

    /* clear pending DIO changed bits by reading register */
    uint32 u32IntSource = u32AHI_DioWakeStatus();
    if (u32IntSource & BUTTON_MASK) {
       /* disable edge detection until scan complete */
        vAHI_DioWakeEnable(0, BUTTON_MASK);
        DBG_vPrintf(TRACE_APP, "ISR\n");

        // Activate button scan task is 5ms
        OS_eStartSWTimer(APP_tmrButtonScan, APP_TIME_MS(5), NULL);
    }

}


PRIVATE void vWriteToPortPin(uint8 u8PinNo, bool bState) {
    if (u8PinNo < 21) {
        if (bState) {
            vAHI_DioSetOutput((1<<u8PinNo), 0  );
        } else {
            vAHI_DioSetOutput(0, (1<<u8PinNo));
        }
    }
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
